#!/usr/dist/pkgs/perl,v5.003/5bin.sun4/perl

require 5.003;
BEGIN {
    my($workdir) = ($0 =~ /(.*)\/.*/);
    unshift(@INC,"$workdir/../lib");
}
my($workdir) = ($0 =~ /(.*)\/.*/);
use BER;
use SNMP_Session;
use SNMP_MIB;
use SNMP_Simple;
use Socket;
use strict;
use Ping;

sub getnet($$) {
    my ($ipaddr, $netmask) = @_;
    my $ipnum = inet_aton($ipaddr);
    my $netmasknum = inet_aton($netmask);
    if (!defined($netmasknum) || !defined($ipaddr)) {
	return $ipaddr;
    }
    my $netnum = $ipnum & $netmasknum;
    return inet_ntoa($netnum);    
}

$SNMP_Session::suppress_warnings = 1;
$SNMP_Simple::suppress_warnings = 1;

SNMP_MIB::loadoids("$workdir/../mibs", "RFC1213");
### MAIN ###

my $ping = Ping->new("icmp");
my $host = "$ARGV[0]";
my $netmask = "$ARGV[1]";
my $community = "public";
my $basedir = "/var/tmp";
my $hostip = gethostbyname($host);
#$hostip = inet_aton($hostip);
my ($aip,$bip,$cip,$dip) = unpack('C4',$hostip);
$| = 1;

open(LOG, ">/tmp/discover.log");
autoflush LOG 1;

my (%routerlist, %routervisited, @worklist, $curhost);
$routerlist{"$aip.$bip.$cip.$dip"} = $host;

print "graph SWAN { \n";

print LOG "\nDiscover started at ".time."\n";

open (RTR, ">/tmp/discover.routers");
autoflush RTR 1;

for ($curhost = "$aip.$bip.$cip.$dip";
     defined($curhost); $curhost = pop @worklist) {

    my $hostname = gethostbyaddr(pack('C4', split(/\./, $curhost)), AF_INET);
    
    next if (defined($routervisited{$curhost}));
    
    print LOG "Visiting $curhost\n";
    $routervisited{$curhost} = 1;

    if ($ping->ping($curhost)) {
	my $session = SNMP_Simple->open($curhost, $community, 161);

        if (!defined($session)) {
	    print LOG "Failed open of SNMP session to host $curhost\n";
	    next;
	}

	$session->{'session'}->{'retries'} = 5;

	# Get basic system info
	my ($sysName, $sysContact, $sysLocation, $sysObjectID, $router) 
	    = $session->snmpget('sysName', 'sysContact', 'sysLocation', 
				'sysObjectID', 'ipForwarding');

	if (!defined($sysName) || ($sysName eq "")) {
	    print LOG "$curhost does not have sysName defined\n";
	    $sysName = $curhost;
	}

	# Check if it is SNMP enabled and is a router.
	if (defined($router) && ($router == 1)) {
	    print RTR "$curhost\n";
	    
	    # Get IP address table entries.
	    my @rtrips = $session->snmpgettable('ipAdEntAddr');
	    my %ipindicies = $session->snmpgettablea('ipAdEntIfIndex');
	    my %netmasks = $session->snmpgettablea('ipAdEntNetMask');
	    my $indexoid = SNMP_MIB::name2oid('ipAdEntIfIndex');
	    my $netoid = SNMP_MIB::name2oid('ipAdEntNetMask');
	    my %intsvisited;
	    my $rtrip;

	    print "\t\"$sysName\" -- {";

	    # Iterate through the IP addresses, and make a graph
	    # connection for each.  Mark the interface as visited, 
	    # so we do not make another connection for that interface
	    # later when we are looking at the routes.

	    foreach $rtrip (@rtrips) {
		my $netmask = $netmasks{"$netoid.$rtrip"};
		my $net = getnet($rtrip, $netmask);
		my $index = $ipindicies{"$indexoid.$rtrip"};

		$intsvisited{$index} = 1;

		# check if this is the loopback net (127.0.0.0/8)
		# and go on if it is.
		my $loopback = getnet($rtrip, "255.0.0.0");
		next if ($loopback eq "127.0.0.0");

		$routerlist{$rtrip} = $sysName;

		# If we have not already visited this IP, mark it as
		# visited.
		if (!defined($routervisited{$rtrip})) {
		    $routervisited{$rtrip} = 1;
		}

		print "\"$net\" ; ";
	    }

	    # Get route table information.
	    my @routedests = $session->snmpgettable('ipRouteDest');
	    my %routeindicies = $session->snmpgettablea('ipRouteIfIndex');
	    my %nexthops = $session->snmpgettablea('ipRouteNextHop');
	    my $routeoid = SNMP_MIB::name2oid('ipRouteIfIndex');
	    my $hopoid = SNMP_MIB::name2oid('ipRouteNextHop');
	    my ($dest, %routers, %uniqrouters);

	    # Iterate through the route table.  For each entry
	    foreach $dest (@routedests) {
		my $routeindex = $routeindicies{"$routeoid.$dest"};
		my $nextrouter = $nexthops{"$hopoid.$dest"};
		my $checknet = getnet($dest, "225.0.0.0");
		my $nextsysName;
		
		if (!defined($nextrouter) || ($nextrouter eq "")) {
		    print LOG "next hop from $curhost to $dest is not defined\n";
		    next;
		}
		
		# Ignore the route if it is to the loopback net, if it
		# is a default route, or if it goes back to this host.
		next if (($checknet eq "127.0.0.0") || 
			 ($nextrouter eq "0.0.0.0") ||
			 ($nextrouter eq $curhost));

		# Check if this list is on the global list of routers,
		# and add it if it is not.
		if (!defined($routerlist{$nextrouter})) {
		    $routerlist{$nextrouter} = $nextrouter;
		    push @worklist, $nextrouter;
		    print LOG "Added $nextrouter to the worklist\n";
		}

		# If we have already seen this interface before when
		# looking at IP addresses, don't bother to do anything
		# more.  Also, if we have already seen this router,
		# don't do anything more.
		
		if (!defined($intsvisited{$routeindex})
		    && !defined($routers{$nextrouter})) {

		    my $nextsession = SNMP_Simple->open($nextrouter, 
							$community, 161);

		    # Bail and go to the next destination if
		    # we can not open a session.
		    next if (!defined($nextsession));

		    ($nextsysName) = $nextsession->snmpget('sysName');

		    # Check for a valid return.
		    if (defined($nextsysName) && ($nextsysName ne "")) {
			$routers{$nextrouter} = $nextsysName;

			# Check to see if the system name has
			# been seen before, if not print it out.
			if (!defined($uniqrouters{$nextsysName})) {
			    if ($nextsysName ne $sysName) {
				print "\"$nextsysName\" ; ";
			    }
			    $uniqrouters{$nextsysName} = 1;
			}

		    } else {
			$nextsession->close();
			next;
		    }
		    $nextsession->close();
		}

	    } # end foreach $dest
	    print "}\n";

	} else { # router not defined
	    print LOG "$curhost does not respond to SNMP or is not a router\n";
	} # end if (!defined($router) ...

	$session->close();
    } else { # can not ping host
	print LOG "$curhost not reachable\n";
    } # end if ($ping->ping($curhost))

} # end for $curhost

$ping->close();
close(LOG);
close(RTR);

print "}\n";

exit 0;
