#!/usr/dist/pkgs/perl,v5.003/5bin.sun4/perl

require 5.003;
BEGIN {
	my($workdir) = ($0 =~ /(.*)\/.*/);
	unshift(@INC,"$workdir/../lib");
}

my($workdir) = ($0 =~ /(.*)\/.*/);
use BER;
use SNMP_Session;
use SNMP_MIB;
use SNMP_Simple;
use Socket;
use strict;

package Element;

sub new () {
    my $type = shift;
    my $self = {};
    bless $self, $type;
}

package main;

### MAIN ###

my($host) = $ARGV[0];
my($router) = $ARGV[1];
my(@hostip);
my($community) = "public";

$SNMP_Session::suppress_warnings = 1;
SNMP_MIB::loadoids("$workdir/../mibs", "RFC1213", "BRIDGE", "IF", "CISCO-STACK");

@hostip = gethostbyname($host);

die "$host not a valid hostname" 
    unless defined ($hostip[0]);

$router = $host
    unless defined(gethostbyname($router));


$hostip[0] = inet_aton($hostip[0]);
my ($aip,$bip,$cip,$dip) = unpack('C4',$hostip[0]);
my $origaddr = "$aip.$bip.$cip.$dip";
my (@routers, @bridges);
my ($netmask, $physaddr, $physaddrstr, $physaddroid);

my $rtrsession = SNMP_Simple->open($router, $community, 161);

my $rtrtest = $rtrsession->snmpget('ipForwarding');

$rtrsession = SNMP_Simple->open($host, $community, 161)
    unless (defined($rtrtest) && $rtrtest == 1);

my (%netAddress) = $rtrsession->snmpgettablean("atPhysAddress");
$rtrsession->close();

my $session = SNMP_Simple->open($host, $community, 161);

my (%phys) = $session->snmpgettablean("dot1dTpFdbPort");
my (%dot1dtoif) = $session->snmpgettablean("dot1dBasePortIfIndex");

my (%ifs) = $session->snmpgettableka(1, "ifDescr", "ifName");
my (@vlans) = $session->snmpgettable("vlanIndex");


$session->close();

my ($port);
my (%vlanh);
foreach $port (keys %dot1dtoif) {
    $vlanh{$port} = 1;
}

my $vlan;
foreach $vlan (@vlans) {
    my $dot1dsession = SNMP_Simple->open($host, "$community\@$vlan", 161);
    my (%lphys) = $dot1dsession->snmpgettablean("dot1dTpFdbPort");
    my (%ldot1dtoif) = $dot1dsession->snmpgettablean("dot1dBasePortIfIndex");
    my ($tmp);
    foreach $tmp (keys %lphys) {
	if (defined($phys{$tmp})) {
	    #print "Addr: $tmp in $phys{$tmp} != $lphys{$tmp}\n" 
	    #if ($phys{$tmp} != $lphys{$tmp});
	} else {
	    $phys{$tmp} = $lphys{$tmp};
	}
    }
    foreach $tmp (keys %ldot1dtoif) {
	if (defined($dot1dtoif{$tmp})) {
	    #print "Port: $tmp in $dot1dtoif{$tmp} != $ldot1dtoif{$tmp}\n"
	    #	if ($dot1dtoif{$tmp} != $ldot1dtoif{$tmp});
	    #print "Vlan: $tmp in $vlanh{$tmp}, $vlan\n"
	    $vlanh{$tmp} = "*"
		if $vlanh{$tmp} != $vlan;
	} else {
	    $dot1dtoif{$tmp} = $ldot1dtoif{$tmp};
	    $vlanh{$tmp} = $vlan;
	}
    }
    $dot1dsession->close();
}

my (%phystonet);
my ($physaddress, $netkey);
my $hostname;

foreach $netkey (keys %netAddress) {
    $physaddress = $netAddress{$netkey};
    my (@physaddrarr) = split(/\./, $netkey);
    shift @physaddrarr; shift @physaddrarr;
    my $physaddrstr = sprintf("%d.%d.%d.%d.%d.%d", unpack('C6', $physaddress));
    $phystonet{$physaddrstr} = sprintf("%d.%d.%d.%d", @physaddrarr);
#    print "Mac: $physaddrstr -> $phystonet{$physaddrstr} $netkey\n";
}

my ($ifIndex, $physport, $ifName, $vlan, $ifDescr, $physaddrstr, $netAddr);

format DATA_TOP =
Int Port Name  VLAN Description                   MAC                IP
--------------------------------------------------------------------------------          
.

format DATA = 
@<< @<<< @<<<< @<<< @<<<<<<<<<<<<<<<<<<<<<<<<<<<< @<<<<<<<<<<<<<<<<< @<<<<<<<<<<<<<<< @<<<<<<<<<<<<<<<<<<<<<<<<<<
$ifIndex, $physport, $ifName, $vlan, $ifDescr, $physaddrstr, $netAddr, $hostname
.

$~ = "DATA";

my (%elements);

foreach $physaddress (keys %phys) {
    my $element = Element->new();
    $element->{'physaddr'} = 
	sprintf("%02x:%02x:%02x:%02x:%02x:%02x", split(/\./, $physaddress));
    my $physport = $phys{$physaddress};
    $element->{'physport'} = $physport;
    $element->{'vlan'} = $vlanh{$physport};
    my $ifIndex = $dot1dtoif{$physport};
    $element->{'ifIndex'} = $ifIndex;
    $element->{'ifName'} =  $ifs{$ifIndex}->{'ifName'};
    $element->{'ifDescr'} = $ifs{$ifIndex}->{'ifDescr'};
    $element->{'netAddr'} = $phystonet{$physaddress};
    push(@{$elements{$ifIndex}},$element);
}

sub numerically { $a <=> $b; }

my $i;
foreach $i (sort numerically keys(%elements)) {
    my $element;
    foreach $element (@{$elements{$i}}) {
	$physaddrstr = $element->{'physaddr'};
	$physport = $element->{'physport'};
	$ifName = $element->{'ifName'};
	$vlan = $element->{'vlan'};
	$ifIndex = $element->{'ifIndex'};
	$ifDescr = $element->{'ifDescr'};
	$netAddr = $element->{'netAddr'};
	$hostname = gethostbyaddr(pack('C4', split(/\./, $netAddr)),
AF_INET);
	write;
    }
}

exit 0;

