#!/usr/local/bin/perl
#
#    mibdump.pl - NMIS CGI Program - Network Mangement Information System
#    Copyright (C) 2000 Sinclair InterNetworking Services Pty Ltd
#    <nmis@sins.com.au> http://www.sins.com.au/nmis
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#*****************************************************************************
# Auto configure to the <nmis-base>/lib and <nmis-base>/files/nmis.conf
use FindBin;
use lib "$FindBin::Bin/../lib";

# 
#****** Shouldn't be anything else to customise below here *******************

# Check if there are any arguements
if ( $#ARGV < 0 ) {
        print "$0 NMIS MIB Dumping Tool\n";
        print "command line options are:\n";
        print "\tmibdir=<directory>     Location of the MIBS\n";
        print "\tmibs=<standard|router|switch|server>]  Which MIBs to load\n";
        print "\toutfile=<filename>]    Where to dump the OIDs to.\n";
        print "\n";
        
        exit(1);
}

use strict;
use SNMP_MIB;
use func;
use NMIS;

my %ARGS = getArguements(@ARGV);

my $conf;
if ( $ARGS{file} ne "" ) { $conf = $ARGS{file}; }
else { $conf = "nmis.conf"; }
my $configfile = "$FindBin::Bin/../conf/$conf";
if ( -f $configfile ) { loadConfiguration($configfile); }
else { die "Can't access configuration file $configfile.\n"; }
								       
if ( ! defined $ARGS{mibdir} ) { $ARGS{mibdir} = "/usr/local/share/snmp/mibs"; }

# Always load the default mib
SNMP_MIB::loadmib($ARGS{mibdir}, "RFC1213-MIB.my");

&loadMibs();
&dumpOID();

sub loadMibs {
	print "Loading MIBS from $ARGS{mibdir}\n";
	if ( $ARGS{mibs} eq "all" ) { 
		# Standard Mibs
		SNMP_MIB::loadmib($ARGS{mibdir}, "IF-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "IANAifType-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "ETHERLIKE-MIB.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "RFC1315-MIB.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "SNMPv2-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "SNMPv2-SMI-V1SMI.my ");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "SNMPv2-TC-V1SMI.my ");
		SNMP_MIB::loadmib($ARGS{mibdir}, "BRIDGE-MIB.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "ENTITY-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-SMI-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-PRODUCTS-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-STACK-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-PROCESS-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-MEMORY-POOL-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-ENVMON-MIB-V1SMI.MY");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-FRAME-RELAY-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-FIREWALL-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "OLD-CISCO-SYS-MIB.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "OLD-CISCO-CHASSIS-MIB.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "OLD-CISCO-INTERFACES-MIB.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "UCD-SNMP-MIB.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "sun.mib");
		SNMP_MIB::loadmib($ARGS{mibdir}, "snmpdx.mib");
		SNMP_MIB::loadmib($ARGS{mibdir}, "msft.mib");
		SNMP_MIB::loadmib($ARGS{mibdir}, "lmmib2.mib");
		#SNMP_MIB::loadmib($ARGS{mibdir}, "MS-Perfmib.mib");
		SNMP_MIB::loadmib($ARGS{mibdir}, "HOST-RESOURCES-MIB.txt");
		# Nortel MIBS?
		SNMP_MIB::loadmib($ARGS{mibdir}, "ACCELAR-MIB.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "SYNRO172-MIB.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "S5REG180-MIB.txt");
	}
	elsif ( $ARGS{mibs} eq "standard" ) { 
		# Standard Mibs
		SNMP_MIB::loadmib($ARGS{mibdir}, "IF-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "IANAifType-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "ETHERLIKE-MIB.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "RFC1315-MIB.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "ENTITY-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "SNMPv2-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "SNMPv2-SMI-V1SMI.my ");
		
	}
	elsif ( $ARGS{mibs} eq "switch" ) { 
		# Cisco Switch MIBS
		SNMP_MIB::loadmib($ARGS{mibdir}, "BRIDGE-MIB.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-SMI-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-PRODUCTS-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-STACK-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-PROCESS-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-MEMORY-POOL-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-ENVMON-MIB-V1SMI.MY");
		SNMP_MIB::loadmib($ARGS{mibdir}, "OLD-CISCO-INTERFACES-MIB.my");
	}
	elsif ( $ARGS{mibs} eq "router" ) { 
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-SMI-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-PRODUCTS-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "OLD-CISCO-SYS-MIB.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "OLD-CISCO-CHASSIS-MIB.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "OLD-CISCO-INTERFACES-MIB.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-PROCESS-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-MEMORY-POOL-MIB-V1SMI.my");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-ENVMON-MIB-V1SMI.MY");
		SNMP_MIB::loadmib($ARGS{mibdir}, "CISCO-FRAME-RELAY-MIB-V1SMI.my");
	}	
	elsif ( $ARGS{mibs} eq "server" ) { 
		# Cisco Switch MIBS
		SNMP_MIB::loadmib($ARGS{mibdir}, "UCD-SNMP-MIB.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "sun.mib");
		SNMP_MIB::loadmib($ARGS{mibdir}, "sun-snmpdx.mib");
		SNMP_MIB::loadmib($ARGS{mibdir}, "MS-LMMIB2.MIB");
		SNMP_MIB::loadmib($ARGS{mibdir}, "MS-Perfmib.mib");
		SNMP_MIB::loadmib($ARGS{mibdir}, "HOST-RESOURCES-MIB.txt");
		SNMP_MIB::loadmib($ARGS{mibdir}, "HOST-RESOURCES-TYPES.txt");
	}
}

sub dumpOID {
	open (STDOUT,">$NMIS::config{mib_root}/$ARGS{outfile}") or die "ERROR, problem opening $NMIS::config{mib_root}/$ARGS{outfile}. $!\n";

	SNMP_MIB::dump_oids_file();

	close (STDOUT);
}

exit(1);
