#!/usr/local/bin/perl
#
#    view.pl - NMIS View Tables Program - Network Mangement Information System
#    Copyright (C) 2001 Sinclair InterNetworking Services Pty Ltd 
#    <nmis@sins.com.au> http://www.sins.com.au/nmis
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#*****************************************************************************
# Auto configure to the <nmis-base>/lib and <nmis-base>/files/nmis.conf
use FindBin;
use lib "$FindBin::Bin/../lib";
use lib "/usr/local/rrdtool/lib/perl"; 

# 
#****** Shouldn't be anything else to customise below here *******************

use strict;
use web;
use NMIS;
use func;
use csv;

my $scriptname = $ENV{SCRIPT_NAME};

# This reads in the information sent when the user pressed Submit
my %FORM = getCGIForm($ENV{'QUERY_STRING'});

# Allow program to use other configuration files
my $conf;
if ( $FORM{file} ne "" ) { $conf = $FORM{file}; }
else { $conf = "nmis.conf"; }
my $configfile = "$FindBin::Bin/../conf/$conf";
if ( -f $configfile ) { loadConfiguration($configfile); }
else { die "Can't access configuration file $configfile.\n"; }

# Break the queary up for the names
my $type = $FORM{type};

my $counter;
my $group;

my $colspan = 5;

my %types;
my %categories;
my @fields;
my $nmis_url = "<a href=\"$NMIS::config{nmis}?file=$conf\"><img alt=\"NMIS Dash\" src=\"$NMIS::config{nmis_icon}\" border=\"0\"></a>";
my $back_url = "<a href=\"$ENV{HTTP_REFERER}\"><img alt=\"Back\" src=\"$NMIS::config{back_icon}\" border=\"0\"></a>";
my $help_url = "<a href=\"$NMIS::config{help_file}#$FORM{table}\"><img alt=\"Help\" src=\"$NMIS::config{help_icon}\" border=\"0\"></a>";

if ( ( $FORM{edit} eq "delete" or $FORM{edit} ne "" or $FORM{add} eq "true" ) and $FORM{change} ne "true" ) {
        &editRowMenu(table => $FORM{table}, row => $FORM{row}, edit => $FORM{edit});
}
elsif ( $FORM{edit} ne "" and $FORM{change} eq "true" ) {
        &editRow(table => $FORM{table}, row => $FORM{row}, edit => $FORM{edit} );
        &displayTable(table => $FORM{table}, name => $FORM{name}, column => $FORM{column}, value => $FORM{value});
}
elsif ( $FORM{table} ne "" ) {
        &displayTable(table => $FORM{table}, name => $FORM{name}, column => $FORM{column}, value => $FORM{value});
}
else {
}

exit;

sub editRow {
	my %args = @_;
	my $table = $args{table};
	my $row = $args{row};
	my $edit = $args{edit};
	my $name = $args{name};
	my $column = $args{column};
	my $value = $args{value};

	my $field;
	my @headers;
	my $head;
	my $i;
	my $tmp;
	my $table_file = $table."_Table";
	my $key = $table."_Key";
	my %table_data = loadCSV($NMIS::config{$table_file},$NMIS::config{$key});

	if ( $edit ne "delete" ) {
		if ( $row eq "add" and $edit eq "true" ) {
			# AS 09.05.02 Ambrose Li's sort patch
			foreach $key (sort alphanumerically (keys %table_data)) {
				if ( not defined $tmp ) { $tmp = $table_data{$key}; }
			}
		}
		elsif ( $row ne "" and $edit ne "add" ) {
			$tmp = $table_data{$row};
		}
		# need to get a row to build the menu.
                        # AS 09.05.02 Ambrose Li's sort patch
		foreach $field (sort alphanumerically (keys %$tmp)) {
			$table_data{$row}{$field} = $FORM{$field};
			#print STDERR returnTime." editRow, field=$field row=$row table=$table_data{$row}{$field}\n";
		}
	}
	else {
		delete $table_data{$row};
	}
	&writeCSV(%table_data,$NMIS::config{$table_file},"\t");
}

sub editRowMenu {
	my %args = @_;
	my $table = $args{table};
	my $row = $args{row};
	my $edit = $args{edit};
	my $name = $args{name};
	my $column = $args{column};
	my $value = $args{value};
	
	my $short_table = $table;
	my $field;
	my $tmp_key;
	my $header;
	my $header2;
	my $wide = 2;
	my $counter = 1;
	my $pass = 1;
	my @headers;
	my $i;
	my $head;
	my $tmp;
	my $colspan;
	my $button;
	
	my $table_file = $table."_Table";
	my $key = $table."_Key";
	$tmp = $table."_Title";
	$header = "NMIS $NMIS::config{$tmp}";
	$header2 = "$back_url$nmis_url$help_url NMIS $NMIS::config{$tmp}";

	my %table_data = loadCSV($NMIS::config{$table_file},$NMIS::config{$key});
	#@headers = loadCSVHeaders($NMIS::config{$table_file});

	if ( $FORM{debug} eq "true") { ++$colspan; }
	$short_table =~ s/s$//g;
	pageStart($header,$NMIS::config{style});
	cssTableStart("plain");

	print "<tr><td class=\"view_header\" colspan=\"2\">$header2</td></tr>\n";

	# make the header the row.
	if ( $row ne "" ) {
		$button = "EDIT";
		$tmp_key = $table_data{$row};
	}
	# need to get a row to build the menu.
	else {
		$button = "ADD";
		$row = "add";
		foreach $key  ( keys %table_data ) {
			if ( not defined $tmp_key ) { $tmp_key = $table_data{$key}; }
		}
	}
	$i = 0;
                        # AS 09.05.02 Ambrose Li's sort patch
	foreach $head (sort alphanumerically (keys %$tmp_key)) {
		$headers[$i] = $head;
		++$i;
	}
	
	print "<form action=\"$ENV{SCRIPT_NAME}\" method=\"get\">\n";
	foreach $field (@headers) {
		print <<EO_HTML;
	<tr>
		<td class="head">$field</td>
		<td class="view"><input type="text" size="40" name="$field" value="$table_data{$row}{$field}"></td>
	</tr>
EO_HTML
	}
	print "<input type=\"hidden\" name=\"table\" value=\"$table\">\n";
	print "<input type=\"hidden\" name=\"row\" value=\"$row\">\n";
	print "<input type=\"hidden\" name=\"edit\" value=\"true\">\n";
	print "<input type=\"hidden\" name=\"change\" value=\"true\">\n";
	print "<tr><td class=\"menugrey\" colspan=\"2\"><input type=\"submit\" value=\"$button\"></td></tr>\n";
	print "</form>\n";
	if ( $edit eq "delete" ) {
		print "<form action=\"$ENV{SCRIPT_NAME}\" method=\"get\">\n";
		print "<input type=\"hidden\" name=\"table\" value=\"$table\">\n";
		print "<input type=\"hidden\" name=\"row\" value=\"$row\">\n";
		print "<input type=\"hidden\" name=\"edit\" value=\"delete\">\n";
		print "<input type=\"hidden\" name=\"change\" value=\"true\">\n";
		print "<tr><td class=\"menugrey\" colspan=\"2\"><input type=\"submit\" value=\"DELETE\"></td></tr>\n";
		print "</form>\n";
	}
	tableEnd;
	pageEnd;
}

sub displayTable {
	my %args = @_;
	my $table = $args{table};
	my $name = $args{name};
	my $column = $args{column};
	my $value = $args{value};
	
	my $short_table = $table;
	my $field;
	my $header;
	my $header2;
	my $wide = 2;
	my $counter = 1;
	my $pass = 1;
	my @headers;
	my $i;
	my $c;
	my $tmp;
	my $head;
	my $key;
	my $got_header = 0;
	my $tmp;
	my $colspan;
	my @key_head;
	
	my $table_file = $table."_Table";
	my $table_key = $table."_Key";
	$tmp = $table."_Title";
	$header = "NMIS $NMIS::config{$tmp}";
	$header2 = "$back_url$nmis_url$help_url NMIS $NMIS::config{$tmp}";

	my %table_data = loadCSV($NMIS::config{$table_file},$NMIS::config{$table_key});
	#@headers = loadCSVHeaders($NMIS::config{$table_file});
	my $colspan = $#headers + 1;

	if ( $FORM{debug} eq "true") {	
       		++$colspan;
       	}
       	if ( $name ne "" ) {
		$short_table =~ s/s$//g;
       		$header = "$short_table $name";
       	}
       	if ( $value ne "" ) {
       		$header = $header." - Records for $column=$value";
	}        
	pageStart($header,$NMIS::config{style});
	cssTableStart("white");

	#Display each data Row
	# AS 09.05.02 Ambrose Li's sort patch
	foreach $key ( sort alphanumerically (keys %table_data) ) {
		print "\n";
		++$counter;
		++$pass;
		#Get the headers populated quick!
		if ( not $got_header ) {
			$got_header = 1;
			
			#put the header in order with $key at front.
			@key_head = split(/:/,$NMIS::config{$table_key});
			$c = 0;
			for $i (0..$#key_head) {
				$headers[$i] = $key_head[$i];
				++$c;
			}
	
			$i = $c;
			foreach $head (sort alphanumerically (keys %{$table_data{$key}})) {
				if ( $NMIS::config{$table_key} !~ /$head/ ) {
					$headers[$i] = $head;
					++$i;
				}
			}
			
			# Page Start Stuff!
			$colspan = $#headers + 2;
			print "<tr><td class=\"view_header\" colspan=\"$colspan\">$header2</td></tr>\n";
			if ( $FORM{debug} eq "true" ) {
				print "<tr><td class=\"view\" colspan=\"$colspan\">table=$table_file $NMIS::config{$table_file} key=$key $NMIS::config{$key}</td></tr>\n";	
			}

			#Display Header Row
			## AS 8 June 2002 - Implemented changes for view control.
			rowStart;
			if ( $FORM{debug} eq "true" ) {
				cssPrintCell("head","key",1);
			}
			foreach $field (@headers) {
				cssPrintCell("head","$field",1);
			}
 			my $add="<a href=\"$scriptname?file=$conf&table=$FORM{table}&add=true\">Add</a>";
 			if ($ENV{REMOTE_USER} eq "view") {
 				$add="View Only" ;
 			}
 			cssPrintCell("head",$add,1);
 			rowEnd;
		} # if ( not $got_header )		

		if ( 	($name eq "" and $value eq "") 
			or ( lc($name) eq lc($key) ) 
			or ( $value ne "" and $value eq $table_data{$key}{$column}) 
		) {
			rowStart;
		}
		if ( $FORM{debug} eq "true") {	
			cssPrintCell("view","$key",1);
		}
		for $i (0..$#headers) {
			$field = $headers[$i];
			if ( $name eq "" and $value eq $table_data{$key}{$column} ) {
				cssPrintCell("view","$table_data{$key}{$field}",1);
			}
			elsif ( $name eq "" and $value eq "" ) {
				cssPrintCell("view","$table_data{$key}{$field}",1);
			}
			elsif ( $name ne "" and lc($name) eq lc($key) ) {
				cssPrintCell("view","$table_data{$key}{$field}",1);
			}
		}
		if ( 	($name eq "" and $value eq "") 
			or (lc($name) eq lc($key)) 
			or ( $value ne "" and $value eq $table_data{$key}{$column}) 
		) {
			my $editdelete="<a href=\"$scriptname?file=$conf&table=$FORM{table}&row=$key&edit=true\">Edit</a>"
			."<a href=\"$scriptname?file=$conf&table=$FORM{table}&row=$key&edit=delete\">Delete</a>";

			if ($ENV{REMOTE_USER} eq "view") {
			     $editdelete="View Only" ;
			}
			cssPrintCell("button",$editdelete,1);
			rowEnd;
		}
	}
	tableEnd;
	pageEnd;
}

# AS 09.05.02 Adding Ambrose Li's sort patch
sub alphanumerically {
	 local($&, $`, $', $1, $2, $3, $4);
	 # Sort numbers numerically
	 return $a <=> $b if $a !~ /\D/ && $b !~ /\D/;
	 # Sort IP addresses numerically within each dotted quad
	 if ($a =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/) {
	 	my($a1, $a2, $a3, $a4) = ($1, $2, $3, $4);
	 	if ($b =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/) {
	 		my($b1, $b2, $b3, $b4) = ($1, $2, $3, $4);
	 		return ($a1 <=> $b1) && ($a2 <=> $b2)
	 		&& ($a3 <=> $b3) && ($a4 <=> $b4);
	 	}
	 }
	 # Handle things like Level1, ..., Level10
	 if ($a =~ /^(.*\D)(\d+)$/) {
	 	my($a1, $a2) = ($1, $2);
	 	if ($b =~ /^(.*\D)(\d+)$/) {
	 		my($b1, $b2) = ($1, $2);
	 		return $a2 <=> $b2 if $a1 eq $b1;
	 	}
	 }
	 # Default is to sort alphabetically
	 return $a cmp $b;
}
